;*********************************************************************
; JoyKontrol - Analog joystick to MIDI converter                     *
;*********************************************************************
; (c) 2005 SM-Sound / Seb Francis                                    *
;*********************************************************************
;
; Memory map, and constant declarations
;
;*********************************************************************


;==================================================
; MISC

#DEFINE			_C	STATUS,C
#DEFINE			_Z	STATUS,Z
LSB			EQU	0x0
MSB			EQU	0x7

DIGIT1			EQU	0x0
DIGIT2			EQU	0x1
DIGIT3			EQU	0x2
DP			EQU	0x7

EDIT			EQU	0x4
MEMORY			EQU	0x5

DEBUG1			EQU	0x3
DEBUG2			EQU	0x4
DEBUG3			EQU	0x5


;==================================================
; Memory - bank 0

; Min			EQU	0x0020

MEMORY_LOCATION		EQU	0x0020	; Currently active memory location (1-5)

MIDI_CHANNEL		EQU	0x0021	; MIDI transmit channel (0-15)

; Amount to subtract from controllers mid-points to get 0 (8 bit unsigned) - defaults to 0x80
OFFSET_X1		EQU	0x0022
OFFSET_Y1		EQU	0x0023
OFFSET_Z1		EQU	0x0024
OFFSET_X2		EQU	0x0025
OFFSET_Y2		EQU	0x0026
OFFSET_Z2		EQU	0x0027

; Last values sent by controllers (7 bit unsigned)
LAST_X1_POS		EQU	0x0028
LAST_Y1_POS		EQU	0x0029
LAST_Z1_POS		EQU	0x002A
LAST_X2_POS		EQU	0x002B
LAST_Y2_POS		EQU	0x002C
LAST_Z2_POS		EQU	0x002D
LAST_DATA_POS		EQU	0x002E
LAST_X1_NEG		EQU	0x002F
LAST_Y1_NEG		EQU	0x0030
LAST_Z1_NEG		EQU	0x0031
LAST_X2_NEG		EQU	0x0032
LAST_Y2_NEG		EQU	0x0033
LAST_Z2_NEG		EQU	0x0034
LAST_DATA_NEG		EQU	0x0035

; Current values of each A/D channel
CURRENT_X1		EQU	0x0036
CURRENT_Y1		EQU	0x0037
CURRENT_Z1		EQU	0x0038
CURRENT_X2		EQU	0x0039
CURRENT_Y2		EQU	0x003A
CURRENT_Z2		EQU	0x003B
CURRENT_DATA		EQU	0x003C

; Gap 0x003D -> 0x0041

LAST_EDIT_VALUE		EQU	0x0042	; Last value of data pot in edit mode

FLAGS2			EQU	0x0043	; More flags
  SUPPRESS_DATA_KNOB_CHANGE EQU	0x1	; Set when data knob has just been used for changing params and don't want new position sent
  DO_SUPPRESS_NOW	EQU	0x2	; Internally used by SendMIDIMessageIfChanged subroutine
  RESET_LAST_EDIT_VALUE	EQU	0x3	; Indicates LAST_EDIT_VALUE needs to be reset (set when edit mode changes)
  INTERRUPTS_OFF	EQU	0x4	; Used internally by EEPROM reading and writing subroutines

; Current LED display
LED_DIGIT1		EQU	0x0044
LED_DIGIT2		EQU	0x0045
LED_DIGIT3		EQU	0x0046

FLAGS			EQU	0x0047	; Various flags
  EDIT_PRESSED		EQU	0x0	; Set when edit button pressed, clear when released
  MEMORY_PRESSED	EQU	0x1	; Set when memory button pressed, clear when released
  CURRENT_DIGIT_2	EQU	0x2	; If set, current LED digit to update is digit 2
  CURRENT_DIGIT_3	EQU	0x3	; If set, current LED digit to update is digit 3
  MIDI_CC_MSG		EQU	0x4	; Set when MIDI CC message has just been received
  CALIBRATE_OFFSETS	EQU	0x5	; Set to request offsets be calibrated in main loop
  ADVANCED_MODE		EQU	0x6	; Internal flag used by ADConversionToCC subroutine
  ON_DELAY_OVER		EQU	0x7	; Flag set at end of on-delay (1.7s) to enable MIDI transmit and LED display

LED_INTERRUPT_COUNT	EQU	0x0048	; Count of LED interrupts (when this reaches 0, turn LED off)

CALIBRATE_COUNT_L	EQU	0x0049	; LSB of calibrate counter
CALIBRATE_COUNT_H	EQU	0x004A	; MSB of calibrate counter

TEST_MODE_SELECT	EQU	0x004B	; Which controller is being displayed in test mode: 0=None, 1=X1, 2=Y1, 3=Z1, 4=X2, 5=Y2, 6=Z2, 7=DATA

EDIT_MODE_STAGE		EQU	0x004C	; Value indicates stage of edit mode
					; 0=Not in edit mode
					; 1=Controller select
					; 2=Set MIDI channel
					; 3=Simple/Advanced mode select
					; 4=CC select
					; 5=Min select
					; 6=Max select
					; 7=CC select (NEG)
					; 8=Min select (NEG)
					; 9=Max select (NEG)
					; 10=End of edit mode reached (save and exit)
					; 11=End of MIDI channel edit mode reached (save MIDI channel and exit)

EDIT_MODE_CTRL_SELECTED		EQU	0x004D	; Selected controller in edit mode: 0=None, 1=X1, 2=Y1, 3=Z1, 4=X2, 5=Y2, 6=Z2, 7=DATA
EDIT_MODE_CTRL_MODE		EQU	0x004E	; 0=Simple mode, 1=Advanced mode
EDIT_MODE_CC_SELECTED_POS	EQU	0x004F	; Selected POS CC number in edit mode
EDIT_MODE_MIN_SELECTED_POS	EQU	0x0050	; Selected POS min value in edit mode
EDIT_MODE_MAX_SELECTED_POS	EQU	0x0051	; Selected POS max value in edit mode
EDIT_MODE_CC_SELECTED_NEG	EQU	0x0052	; Selected NEG CC number in edit mode
EDIT_MODE_MIN_SELECTED_NEG	EQU	0x0053	; Selected NEG min value in edit mode
EDIT_MODE_MAX_SELECTED_NEG	EQU	0x0054	; Selected NEG max value in edit mode

; Last values sent by controllers in edit mode (7 bit unsigned)
EM_LAST_X1		EQU	0x0055
EM_LAST_Y1		EQU	0x0056
EM_LAST_Z1		EQU	0x0057
EM_LAST_X2		EQU	0x0058
EM_LAST_Y2		EQU	0x0059
EM_LAST_Z2		EQU	0x005A
EM_LAST_DATA    	EQU	0x005B

CC_LEARN		EQU	0x005C	; CC number received by MIDI (0-127, 128 = pitch bend, 129 = aftertouch, 130 = program change, 0xFF = none)
CC_VALUE_LEARN		EQU	0x005D	; CC value received by MIDI (0xFF = no value)

EDIT_MODE_MIDI_CHANNEL	EQU	0x005E	; Selected MIDI channel in edit mode

; Max			EQU	0x005F


; Stuff for fixed point maths routines
; 0x0060-0x006F in bank 0
; 0x00E0-0x00EF in bank 1  - used by interupts, so don't call maths routines in bank 1 from main loop!
; 0x0160-0x016F in bank 2  - memory used for settings, don't call maths routines in bank 2!
; 0x01E0-0x01EF in bank 3
AARGB2			EQU	0x0060		; argument A, LSB
AARGB1			EQU	0x0061		; argument A
AARGB0			EQU	0x0062		; argument A, MSB
BARGB1			EQU	0x0063		; argument B, LSB
BARGB0			EQU	0x0064		; argument B, MSB
REMB1			EQU	0x0065		; remainder LSB
REMB0			EQU	0x0066		; remainder MSB
TEMP			EQU	0x0067
TEMPB3			EQU	0x0068
TEMPB1			EQU	0x0069
TEMPB0			EQU	0x006A
SIGN			EQU	0x006B
LOOPCOUNT		EQU	0x006C
FPFLAGS			EQU	0x006D
  NAN			EQU	0x0
AARGB5			EQU	0x006E		; Used only by UDIV1608 and UDIV1608L macros
SPARE0			EQU	0x006F


; Memory accessible in all banks

W_TEMP        		EQU     0x0070        	; variable used for context saving
STATUS_TEMP		EQU	0x0071		; variable used for context saving
PCLATH_TEMP		EQU	0x0072		; variable used for context saving
FSR_TEMP		EQU	0x0073		; variable used for context saving

; Temp vars for use in interrupts (or during initialisation)
TEMP1			EQU	0x0074
TEMP2			EQU	0x0075
TEMP3			EQU	0x0076
TEMP4			EQU	0x0077
TEMP5			EQU	0x0078
WAITER			EQU	0x0079

; Temp vars for use in main loop (or during initialisation)
MTEMP1			EQU	0x007A
MTEMP2			EQU	0x007B
MTEMP3			EQU	0x007C
MTEMP4			EQU	0x007D
MTEMP5			EQU	0x007E
MWAITER			EQU	0x007F

; Max			EQU	0x007F


;==================================================
; Memory - bank 1

; Min			EQU	0x00A0

; Controller mode flags (bit=0, then simple mode 1 CC per controller; bit=1, then advanced mode 2 CC per controller)
CTRL_MODE		EQU	0x00A0
  X1			EQU	0x0
  Y1			EQU	0x1
  Z1			EQU	0x2
  X2			EQU	0x3
  Y2			EQU	0x4
  Z2			EQU	0x5
  DAT			EQU	0x6
  ; bit 7 is kept at 0 to avoid value ever being 0xFF

; CC messages sent by controllers (0-127, 128 = pitch bend, 129 = aftertouch, 130 = program change, 131 = off)
CC_X1_POS		EQU	0x00A1
CC_Y1_POS		EQU	0x00A2
CC_Z1_POS		EQU	0x00A3
CC_X2_POS		EQU	0x00A4
CC_Y2_POS		EQU	0x00A5
CC_Z2_POS		EQU	0x00A6
CC_DATA_POS		EQU	0x00A7
CC_X1_NEG		EQU	0x00A8
CC_Y1_NEG		EQU	0x00A9
CC_Z1_NEG		EQU	0x00AA
CC_X2_NEG		EQU	0x00AB
CC_Y2_NEG		EQU	0x00AC
CC_Z2_NEG		EQU	0x00AD
CC_DATA_NEG		EQU	0x00AE

; Min CC values sent by controllers (7 bit unsigned)
MIN_X1_POS		EQU	0x00AF
MIN_Y1_POS		EQU	0x00B0
MIN_Z1_POS		EQU	0x00B1
MIN_X2_POS		EQU	0x00B2
MIN_Y2_POS		EQU	0x00B3
MIN_Z2_POS		EQU	0x00B4
MIN_DATA_POS   		EQU	0x00B5
MIN_X1_NEG		EQU	0x00B6
MIN_Y1_NEG		EQU	0x00B7
MIN_Z1_NEG		EQU	0x00B8
MIN_X2_NEG		EQU	0x00B9
MIN_Y2_NEG		EQU	0x00BA
MIN_Z2_NEG		EQU	0x00BB
MIN_DATA_NEG   		EQU	0x00BC

; Max CC values sent by controllers (7 bit unsigned)
MAX_X1_POS		EQU	0x00BD
MAX_Y1_POS		EQU	0x00BE
MAX_Z1_POS		EQU	0x00BF
MAX_X2_POS		EQU	0x00C0
MAX_Y2_POS		EQU	0x00C1
MAX_Z2_POS		EQU	0x00C2
MAX_DATA_POS   		EQU	0x00C3
MAX_X1_NEG		EQU	0x00C4
MAX_Y1_NEG		EQU	0x00C5
MAX_Z1_NEG		EQU	0x00C6
MAX_X2_NEG		EQU	0x00C7
MAX_Y2_NEG		EQU	0x00C8
MAX_Z2_NEG		EQU	0x00C9
MAX_DATA_NEG   		EQU	0x00CA


; Max			EQU	0x00DF

; Stuff for fixed point maths routines
; 0x0060-0x006F in bank 0
; 0x00E0-0x00EF in bank 1
; 0x01E0-0x01EF in bank 3


;==================================================
; Memory - bank 2

; Min			EQU	0x0110


; Max			EQU	0x016F

;==================================================
; Memory - bank 3

; Min			EQU	0x0190


; Max			EQU	0x01DF

; Stuff for fixed point maths routines
; 0x0060-0x006F in bank 0
; 0x00E0-0x00EF in bank 1
; 0x01E0-0x01EF in bank 3


;==================================================
; EEPROM

; Min			EQU	0x00

EE_MEMORY_LOCATION	EQU	0x00	; Currently active memory location (1-5, if 0xFF then assume all EEPROM invalid and load defaults)
EE_MIDI_CHANNEL		EQU	0x01	; MIDI transmit channel (0-15)

; Offsets of controllers mid-points
EE_OFFSET_X1		EQU	0x02
EE_OFFSET_Y1		EQU	0x03
EE_OFFSET_Z1		EQU	0x04
EE_OFFSET_X2		EQU	0x05
EE_OFFSET_Y2		EQU	0x06
EE_OFFSET_Z2		EQU	0x07

; Each memory location is 44 bytes wide
; Start address = mem# * 0x2C - 0x16
EE_MEM1_START		EQU	0x16
  EE_MEM_WIDTH		EQU	0x2C

; Map of memory location 0
EE_CTRL_MODE		EQU	0x16	; If this is 0xFF then assume this memory location invalid and load defaults

EE_CC_X1_POS		EQU	0x17
EE_CC_Y1_POS		EQU	0x18
EE_CC_Z1_POS		EQU	0x19
EE_CC_X2_POS		EQU	0x1A
EE_CC_Y2_POS		EQU	0x1B
EE_CC_Z2_POS		EQU	0x1C
EE_CC_DATA_POS		EQU	0x1D
EE_CC_X1_NEG		EQU	0x1E
EE_CC_Y1_NEG		EQU	0x1F
EE_CC_Z1_NEG		EQU	0x20
EE_CC_X2_NEG		EQU	0x21
EE_CC_Y2_NEG		EQU	0x22
EE_CC_Z2_NEG		EQU	0x23
EE_CC_DATA_NEG		EQU	0x24

EE_MIN_X1_POS		EQU	0x25
EE_MIN_Y1_POS		EQU	0x26
EE_MIN_Z1_POS		EQU	0x27
EE_MIN_X2_POS		EQU	0x28
EE_MIN_Y2_POS		EQU	0x29
EE_MIN_Z2_POS		EQU	0x2A
EE_MIN_DATA_POS		EQU	0x2B
EE_MIN_X1_NEG		EQU	0x2C
EE_MIN_Y1_NEG		EQU	0x2D
EE_MIN_Z1_NEG		EQU	0x2E
EE_MIN_X2_NEG		EQU	0x2F
EE_MIN_Y2_NEG		EQU	0x30
EE_MIN_Z2_NEG		EQU	0x31
EE_MIN_DATA_NEG		EQU	0x32

EE_MAX_X1_POS		EQU	0x33
EE_MAX_Y1_POS		EQU	0x34
EE_MAX_Z1_POS		EQU	0x35
EE_MAX_X2_POS		EQU	0x36
EE_MAX_Y2_POS		EQU	0x37
EE_MAX_Z2_POS		EQU	0x38
EE_MAX_DATA_POS		EQU	0x39
EE_MAX_X1_NEG		EQU	0x3A
EE_MAX_Y1_NEG		EQU	0x3B
EE_MAX_Z1_NEG		EQU	0x3C
EE_MAX_X2_NEG		EQU	0x3D
EE_MAX_Y2_NEG		EQU	0x3E
EE_MAX_Z2_NEG		EQU	0x3F
EE_MAX_DATA_NEG		EQU	0x40

EE_MEM2_START		EQU	0x42
EE_MEM3_START		EQU	0x6E
EE_MEM4_START		EQU	0x9A
EE_MEM5_START		EQU	0xC6
EE_MEM5_END		EQU	0xF1


; Max			EQU	0xFF
